from qiskit import QiskitError
from qiskit_ibm_runtime import QiskitRuntimeService
from qiskit.providers.jobstatus import JobStatus
from json import dumps

from ..utilities import JobResponse

APPLICATION_JSON = 'application/json'


class Job:
    def __init__(self, channel, token, crn, provider_job_id):
        self.channel = channel if channel else "ibm_quantum"
        self.token = token if token else ""
        self.crn = crn if crn else None
        self.job_id = provider_job_id if provider_job_id else ""
        self.retrieve_job = self.__get_job()

    def fetch_job(self) -> JobResponse:

        content_type = None
        job_histogram = None
        job_result = None
        status = self.retrieve_job.status()
        job_status = status.name

        if status == JobStatus.DONE:
            try:
                result = self.retrieve_job.result()
                # case function `sampler`
                if self.retrieve_job.program_id == 'sampler':
                    job_result = result.__dict__
                # case function `circuit-runner`
                else:
                    job_result = result.to_dict()
                    # handle case Statevector object not serializable when run with backend `statevector_simulator`
                    if self.retrieve_job.backend().backend_name == 'statevector_simulator':
                        job_result = dumps(job_result, default=str)
                content_type = APPLICATION_JSON
                # handle circuit has measure
                try:
                    job_histogram = result.get_counts()
                except QiskitError:
                    job_histogram = None
            except Exception as exception:
                job_result = {
                    "error": "Exception when get job result",
                    "exception": str(exception)
                }
                job_status = "ERROR"

        return JobResponse(
            provider_job_id=self.job_id,
            job_status=job_status,
            job_result=job_result,
            content_type=content_type,
            job_histogram=job_histogram
        )

    def __get_job(self):
        service = QiskitRuntimeService(channel=self.channel,
                                       token=self.token,
                                       instance=self.crn)

        return service.job(job_id=self.job_id)
